function [num_cells, max_row] = count_cells(img, thr, min_size, flipv)
% [num_cells, max_row] = COUNT_CELLS(img, thr, min_size, flipv)
% Count number of cells in RGB image img by counting contiguous areas
% larger than or equal to min_size pixels.
% max_row contains the maximum row index with a cell
% flipv is an optional argument. Image is flipped vertically if flipv==1
% so that max_row is measured from the bottom of the image.
%
% The image is converted to binary using threshold thr
%
% R. Gomez-Sjoberg, 2/24/06

if nargin == 3
    flipv = 0;
end

% Eliminate negative pixels
% mask = (img >= 0);
% img = img.*mask;
% Convert to gray scale
if length(size(img)) == 3
    img_gray = rgb2gray(img);
else
    img_gray = img;
end
% Blurr image to remove single bright pixels that slow down the count
img_filt = imfilter(img_gray, ones(5, 5)/25);
img_bin = (img_filt >= thr);
% Identify connected pixels
[L, num_areas] = bwlabeln(img_bin, 4);
% Eliminate areas that are too small
% & find max pixel row position
cc = 0;
if flipv
    mr = 1e6;
else
    mr = 0;
end
for ii = 1:num_areas
    [i, j] = find(L == ii);
    sz = length(i);
    if sz >= min_size
        cc = cc +1;
        if flipv
            min_i = min(i);
            if min_i < mr
                mr = min_i;
            end
        else
            max_i = max(i);
            if max_i > mr
                mr = max_i;
            end
        end
    end
end
num_cells = cc;
if flipv
    si = size(img_bin);
    max_row = si(1) - mr;
else
    max_row = mr;
end
