function sync_folder(srcpath, destpath, verbose)
% SYNC_FOLDER
% Copies all files that are not in the destination path, from the source
% path. If a file exists in the destination path, but is older, it is
% overwritten.
%
% sync_folder(srcpath, destpath, verbose)
%
% srcpath = Source folder
% destpath = Destination foler
% verbose = True/False. Optional argument
%           True = displays name of each file copied.
%           If omitted, it's set to false
%
% R. Gomez-Sjoberg 10/18/05

if nargin == 2
    verbose = false;
end

% Read folder lists
srclist = dir(srcpath);
destlist = dir(destpath);

nsrc = length(srclist);
ndest = length(destlist);
for ii = 1:nsrc
    % Skip . and ..
    if ~strcmp(srclist(ii).name, '.') & ~strcmp(srclist(ii).name, '..')
        % Find entry in the destination folder
        destidx = [];
        for jj = 1:ndest
            if strcmp(srclist(ii).name, destlist(jj).name)
                destidx = jj;
                break;
            end
        end
        if srclist(ii).isdir
            % Entry is a sub-folder
            if isempty(destidx)
                % Create sub-folder if it doesn't exist
                disp(['Creating folder "' destpath '\' srclist(ii).name '"']);
                mkdir([destpath '\' srclist(ii).name]);
            end
            % Sync sub-folders
            sync_folder([srcpath '\' srclist(ii).name], [destpath '\' srclist(ii).name]);
        else
            % Entry is a file
            if ~isempty(destidx)
                % File exists on dest folder, so compare dates
                % Overwrite if destination file is older than source one
                if datenum(srclist(ii).date) > datenum(destlist(destidx).date)
                    write = 1;
                else
                    write = 0;
                end
            else
                write = 1;
            end
            if write
                if verbose
                    disp(['Copying "' srcpath '\' srclist(ii).name '"']);
                end
                dos(['copy "' srcpath '\' srclist(ii).name '" "' destpath '"']);
            end
        end
    end
end
