%%% SequentialTreatments
% Perform sequential chamber operations, to i.e. prepare chip with pluronic
% and fibronectin treatment 
% R. Kellogg
%%%

% Program below will perform pluronic and fibronectin treatment. Assumes:
% PBS connected InPurge (input 0), pluronic to input 1, fibronectin to
% input 2, media to input 3, and 7psi inlet flow pressure
% Step descriptions
% 1. Debubble nonchamber areas with PBS
% 2. Debubble chamber areas with PBS
% 3. Apply pluronic to nonchamber areas, incubate 1 hr
% 4. Wash away pluronic 
% 5. Ensure all pluronic washed by flowing to each Chamber Flush individually
% 6. Apply fibronectin treatment, incubate 8 hr
% 7. Replace fibronectin with media (PBS can also be used)

% Ligand 16 - Media
% Ligand 15 - Media + M-CSF
% Ligand 14 - Media + TNFa
% Ligand 13 - Media + TNFa + M-CSF

%       Step #	1			2		
InputName =     {'Media1',                          'MCSF1',                    'Media2',                   'MCSF2',                        'TNFa2',            'MCSFTNFa2',             'Media3',                      'MCSF3',                        'TNFa3',            'MCSFTNFa3',            'Media4',                   'MCSF4',                        'TNFa4',            'MCSFTNFa4'            'Media5',                   'MCSF5',                        'TNFa5',                     'MCSFTNFa5'                     'Media6',                   'MCSF6',                        'TNFa6',                     'MCSFTNFa6'                            'Incubate'}; % Name of the input reagent
InputNumber =   {16,                                15,                         16,                         15,                             14,                 13,                     16,                             15,                             14,                 13,                     16,                         15,                             14,                 13,                     16,                         15,                             14,                         13,                             16,                         15,                             14,                         13,                                      16}; % Input number on the chip (InPurge PBS = 0)
toChambers =    {1,                                 1,                          1,                          1,                              1,                  1,                      1,                              1,                              1,                  1,                      1,                          1,                              1,                  1,                      1,                          1,                              1,                          1,                              1,                          1,                              1,                          1,                                       0}; % Flow to chambers Yes (1) or No (0)         
toCellsIn =     {0,                                 0,                          0,                          0,                              0,                  0,                      0,                              0,                              0,                  0,                      0,                          0,                              0,                  0,                      0,                          0,                              0,                          0,                              0,                          0,                              0,                          0,                                       0}; % Flow to CellsIn Yes (1) or No (0)       
startFlTime =   {.5,                                .5,                         .5,                         .5,                             .5,                 .5,                     .5,                             .5,                             .5,                 .5,                     .5,                         .5,                             .5,                 .5,                     .5,                         .5,                             .5,                         .5,                             .5,                         .5,                             .5,                         .5,                                      0}; % Duration to flow through Mixer Flush (min) 
incubTime =     {0,                                 0,                          0,                          0,                              0,                  0,                      0,                              0,                              0,                  0,                      0,                          0,                              0,                  0,                      0,                          0,                              0,                          0,                              0,                          0,                              0,                          0,                                       80}; % Wait time after finishing step (min)
flTime =        {20,                                20,                         20,                         20,                             20,                 20,                     20,                             20,                             20,                 20,                     20,                         20,                             20,                 20,                     20,                         20,                             20,                         20,                             20,                         20,                             20,                         20,                                      0}; % Duration to flow through Chamber Flush (sec) 
chTime =        {5,                                 5,                          5,                          5,                              5,                  5,                      5,                              5,                              5,                  5,                      5,                          5,                              5,                  5,                      5,                          5,                              5,                          5,                              5,                          5,                              5,                          5,                                       0}; % Duration to flow through Chambers (sec)  
cinTime =       {0,                                 0,                          0,                          0,                              0,                  0,                      0,                              0,                              0,                  0,                      0,                          0,                              0,                  0,                      0,                          0,                              0,                          0,                              0,                          0,                              0,                          0,                                       0}; % Duration to flow through CellsIn (sec)  
chambers =      {[34,35,38,39,42,43,46,47],         [36,37,40,41,44,45,48,49],  [36,37,42,43,46,47],        [36,37,44,45,48,49],            [38,39],            [40,41],                [34,35,46,47,50,52,54,56],      [36,37,48,49,51,53,55,57],      [38,39,42,43],      [40,41,44,45],          [34,35,46,47,50,54,56],     [36,37,48,49,51,55,57],         [38,39,42,43,52],   [40,41,44,45,53],       [34,35,50,56],              [36,37,51,57],                  [38,39,42,43,46,47,52],     [40,41,44,45,48,49,53,55],      [34,35,50],                 [36,37,51],                     [38,39,42,43,46,47,52,56],  [40,41,44,45,48,49,53,55,57],            97}; % Which chambers to access (97 = all chambers at once)
repeats =       {1,                                 1,                          1,                          1,                              1,                  1,                      1,                              1,                              1,                  1,                      1,                          1,                              1,                  1,                       1,                         1,                              1,                          1,                              1,                          1,                              1,                          1,                                       1}; % How many times to perform this step        
wastesClosed =  {0,                                 0,                          0,                          0,                              0,                  0,                      0,                              0,                              0,                  0,                      0,                          0,                              0,                  0,                       0,                         0,                              0,                          0,                              0,                          0,                              0,                          0,                                       1}; % Chip outlet wastes Open (0) or Closed (1)        
vals = [23,23,3:6,23,23,3:6,23,23,3:6,23,23,3:6,23,23,7:10,23,23,11:14,23,23,11:14,23,23,11:14,23,23,11:14,23,23,15:18,23,23,15:18,23,23,15:18,23,23,15:18,23,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23,19:22,23];  % vals = [3:7] if chip is already debubbled with PBS

for i = vals
    scr_check4stop;
    
    close_valve('ChIn');
    close_valve('ChOut');
    set_mux(99, 1, 1, 1, 0, 1);

    close_inputs;
    close_pump;
    if (InputNumber{i}) == 100
        open_valve('CellsIn')
        open_pump;

    else
        open_inputs(InputNumber{i}) % PBS 
        open_valve('MixerOut');
    end
    open_valve('Flush');
    scr_wait(startFlTime{i}, 0, ['Flush ' InputName{i}]);
    
    close_valve('Flush')
    open_valve('Waste1Dn')
    open_valve('Waste1Up')
    
    % wash CellsIn
    if toCellsIn{i} 
        set_mux(97, 1, 1, 1, 1, 1);
        open_pump;
        open_valve('CellsIn')
        scr_wait(0,cinTime{i}, [InputName{i} ' to CellsIn']);
        close_valve('CellsIn');
    end

    for k = 1:repeats{i}
        scr_check4stop;          
        
        if wastesClosed{i}
            close_valve('Waste1Up');
            close_valve('Waste2Up');
            close_valve('Waste1Dn');
            close_valve('Waste2Dn');
        else
            open_valve('Waste1Up');
            open_valve('Waste1Dn');
        end
            
        for c = chambers{i}
            scr_check4stop;
            set_mux(c, 1, 1, 1, 0, 1);
            open_pump;
            scr_wait(0, flTime{i}, [InputName{i} ' flush']);

            if toChambers{i}
                set_mux(c, 1, 1, 0, 0, 0);
                scr_wait(0, 2, ['Pause for chamber valve']);
                open_valve('ChIn');
                open_valve('ChOut');
                scr_wait(0, chTime{i}, [InputName{i} ' chamber']);
                close_valve('ChIn');
                close_valve('ChOut');
                scr_wait(0, 1, ['Pause for chamber valve']);
                set_mux(99, 1, 1, 0, 0, 0);
            end

        end
        set_mux(99, 1, 1, 1, 0, 1);
        close_pump;
        scr_wait(incubTime{i}, 0, ['Incubating ' InputName{i}]);
    end

end


