%%% SequentialTreatments
% Perform sequential chamber operations, to i.e. prepare chip with pluronic
% and fibronectin treatment 
% R. Kellogg
%%%

% Program below will perform pluronic and fibronectin treatment. Assumes:
% PBS connected InPurge (input 0), pluronic to input 1, fibronectin to
% input 2, media to input 3, and 7psi inlet flow pressure
% Step descriptions
% 1. Debubble nonchamber areas with PBS
% 2. Debubble chamber areas with PBS
% 3. Apply pluronic to nonchamber areas, incubate 1 hr
% 4. Wash away pluronic 
% 5. Ensure all pluronic washed by flowing to each Chamber Flush individually
% 6. Apply fibronectin treatment, incubate 8 hr
% 7. Replace fibronectin with media (PBS can also be used)

%       Step #	1			    2		           3			     4			         5		                6		 7	
InputName =     {'PBShold',     'PBS Flush',   'Pluronic', 'PBS Flush',   '10Hrs',           '8Hrs',             '4Hrs',             '2Hrs',                 'PBS',  'PBSHold'}; % Name of the input reagent
InputNumber =   {1,             1,             2,          1,             3,                 3,                  3,                  3,                      1,      1}; % Input number on the chip (InPurge PBS = 0)
toChambers =    {0,             1,             0,          1,             1,                 1,                  1,                  1,                      0,      0}; % Flow to chambers Yes (1) or No (0)         
toCellsIn =     {0,              0,             0,          0,              0,                 0,                  0,                  0,                      0,      0}; % Flow to CellsIn Yes (1) or No (0)       
startFlTime =   {15,             15,             30,          10,              30,                 5,                  5,                  5,                      5,      0}; % Duration to flow through Mixer Flush (min) 
incubTime =     {0,             0,             0,          240,            111,               231,                111,                106,                    0,      0}; % Wait time after finishing step (min)
flTime =        {60,           30,            30,         30,             30,                30,                 30,                30,                      60, 60*60*16}; % Duration to flow through Chamber Flush (sec) 
chTime =        {60,             60,            0,          60,             60,                60,                 60,                 60,                     0,      0}; % Duration to flow through Chambers (sec)  
cinTime =       {0,             0,             0,          0,              0,                 0,                  0,                  0,                      0,      0}; % Duration to flow through CellsIn (sec)  
chambers =      {97,            13:36,         97,         97,             13:18,             19:24,              25:30,              31:36,                  97,     97}; % Which chambers to access (97 = all chambers at once)
repeats =       {1,             1,             1,          1,              1,                 1,                  1,                  1,                      1,      1}; % How many times to perform this step        
wastesClosed =  {1,             0,             0,          0,              0,                 0,                  0,                  0,                      0,      1}; % Chip outlet wastes Open (0) or Closed (1)        
vals = [1:10];  % vals = [3:7] if chip is already debubbled with PBS

for i = vals
    scr_check4stop;
    
    close_valve('ChIn');
    close_valve('ChOut');
    set_mux(99, 1, 1, 1, 0, 1);

    close_inputs;
    close_pump;
    if (InputNumber{i}) == 100
        open_valve('CellsIn')
        open_pump;

    else
        open_inputs(InputNumber{i}) % PBS 
        open_valve('MixerOut');
    end
    open_valve('Flush');
    scr_wait(startFlTime{i}, 0, ['Flush ' InputName{i}]);
    
    close_valve('Flush')
    open_valve('Waste2Dn')
    open_valve('Waste1Up')
    
    % wash CellsIn
    if toCellsIn{i} 
        set_mux(97, 1, 1, 1, 1, 1);
        open_pump;
        open_valve('CellsIn')
        scr_wait(0,cinTime{i}, [InputName{i} ' to CellsIn']);
        close_valve('CellsIn');
    end

    for k = 1:repeats{i}
        scr_check4stop;          
        
        if wastesClosed{i}
            close_valve('Waste1Up');
            close_valve('Waste2Up');
            close_valve('Waste1Dn');
            close_valve('Waste2Dn');
        else
            open_valve('Waste1Up');
            open_valve('Waste1Dn');
        end
            
        for c = chambers{i}
            scr_check4stop;
            set_mux(c, 1, 1, 1, 0, 1);
            open_pump;
            scr_wait(0, flTime{i}, [InputName{i} ' flush']);

            if toChambers{i}
                set_mux(c, 1, 1, 0, 0, 0);
                scr_wait(0, 2, ['Pause for chamber valve']);
                open_valve('ChIn');
                open_valve('ChOut');
                scr_wait(0, chTime{i}, [InputName{i} ' chamber']);
                close_valve('ChIn');
                close_valve('ChOut');
                scr_wait(0, 1, ['Pause for chamber valve']);
                set_mux(99, 1, 1, 0, 0, 0);
            end

        end
        set_mux(99, 1, 1, 1, 0, 1);
        close_pump;
        scr_wait(incubTime{i}, 0, ['Incubating ' InputName{i}]);
    end

end


