%%% SequentialTreatments
% Perform sequential chamber operations, to i.e. prepare chip with pluronic
% and fibronectin treatment 
% R. Kellogg
%%%



%       Step #	1			2		
InputName =     {'MediaPressure1', 'MediaPressure2',    'MediaPressure3',       'flush'}; % Name of the input reagent
InputNumber =   {9,                 9,                  9,                      9}; % Input number on the chip (InPurge PBS = 0)
toChambers =    {1,                 1,                  1,                      1}; % Flow to chambers Yes (1) or No (0)         
toCellsIn =     {0,                 0,                  0,                      0}; % Flow to CellsIn Yes (1) or No (0)       
startFlTime =   {1,                 1,                  1,                      1}; % Duration to flow through Mixer Flush (min) 
incubTime =     {0,                 0,                  0,                      115}; % Wait time after finishing step (min)
flTime =        {0,                 0,                  0,                      30}; % Duration to flow through Chamber Flush (sec) 
chTime =        {3,                 3,                  3,                      0}; % Duration to flow through Chambers (sec)  
cinTime =       {0,                 0,                  0,                      0}; % Duration to flow through CellsIn (sec)  
chambers =      {4:6,               5:6,                6,                      4:6}; % Which chambers to access (97 = all chambers at once)
repeats =       {100,               100,                100,                    1}; % How many times to perform this step        
wastesClosed =  {1,                 1,                  1,                      0}; % Chip outlet wastes Open (0) or Closed (1)        
vals = [4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4,1,4,2,4,3,4];  % vals = [3:7] if chip is already debubbled with PBS

for i = vals
    scr_check4stop;
    
    close_valve('ChIn');
    close_valve('ChOut');
    set_mux(99, 1, 1, 1, 0, 1);

    close_inputs;
    close_pump;
    if (InputNumber{i}) == 100
        open_valve('CellsIn')
        open_pump;

    else
        open_inputs(InputNumber{i}) % PBS 
        open_valve('MixerOut');
    end
    open_valve('Flush');
    scr_wait(startFlTime{i}, 0, ['Flush ' InputName{i}]);
    
    close_valve('Flush')
    open_valve('Waste1Dn')
    open_valve('Waste1Up')
    
    % wash CellsIn
    if toCellsIn{i} 
        set_mux(97, 1, 1, 1, 1, 1);
        open_pump;
        open_valve('CellsIn')
        scr_wait(0,cinTime{i}, [InputName{i} ' to CellsIn']);
        close_valve('CellsIn');
    end

    for k = 1:repeats{i}
        scr_check4stop;          
        
        if wastesClosed{i}
            close_valve('Waste1Up');
            close_valve('Waste2Up');
            close_valve('Waste1Dn');
            close_valve('Waste2Dn');
        else
            open_valve('Waste1Up');
            open_valve('Waste1Dn');
        end
            
        for c = chambers{i}
            scr_check4stop;
            set_mux(c, 1, 1, 1, 0, 1);
            open_pump;
            scr_wait(0, flTime{i}, [InputName{i} ' flush']);

            if toChambers{i}
                set_mux(c, 1, 1, 0, 0, 0);
                scr_wait(0, 2, ['Pause for chamber valve']);
                open_valve('ChIn');
                open_valve('ChOut');
                scr_wait(0, chTime{i}, [InputName{i} ' chamber']);
                close_valve('ChIn');
                close_valve('ChOut');
                scr_wait(0, 1, ['Pause for chamber valve']);
                set_mux(99, 1, 1, 0, 0, 0);
            end

        end
        set_mux(99, 1, 1, 1, 0, 1);
        close_pump;
        scr_wait(incubTime{i}, 0, ['Incubating ' InputName{i}]);
    end

end


