%%% FeedChambers
% Perform automated cell feeding and stimulation 
% R. Gmez-Sjberg (modifications: R. Kellogg) 
%%% 

% Data path
exp_path = 'C:\CellCultureChip\Experiments\Exp1';
set_path(exp_path);

if exist([exp_path 'StopScript'], 'file')
    beep;
    beep;
    beep;
    infodlg('Title', 'Stop signal!', 'String', ...
        ['WARNING!' 10 13 'Stop signal file ''StopScript'' exists!!!' ...
        10 13 'Please delete before continuing']);
end
   
% Log file name
log_file = 'experiment-log.txt';

% name of FeedChambers file
feed_file = 'FeedChambers';

% Experiment desription
exp_description = {''
''};	

% Run info
run_info = {''};

% Speed factor (used to speed up the whole timing for debugging)
% Should be 1 for a real experiment
SpeedFactor = 1;

% Name of the experimental matrix file
ExpMatrixFile = 'ExpMatrix';
% Name of media schedule file
MediaScheduleFile = 'MediaSchedule';

% Chamber Temperature
ChamberTemperature = 37; %degC

% Number of pump cycles for each condition
% Each condition corresponds to one column in ExpMatrix
% 999 cycles means full replacement
FeedCycles = [999 30];
% Pump cycles for first feeding round
FeedCycles0 = [999 30];
% Pump periods to use with each of the cycle specifications above
FeedPumpPeriods = [0.1]; %seconds
% Alternate forward and backward pumping?
AlternatePumpDir = true;
% Purge MUX at end of feeding?
PurgeMux = false;

% the following options apply if there is no ScriptData.mat file in the
% data folder:
% PrevMediaSchedule supersedes FirstReplace and PrevMedium.
PrevMediaSchedule = true; %if this is true, LastMedia will be initialized based on the media schedule, as rafael designed it.

FirstReplace = false; %if true, then first feeding of every chamber will be full replacement, regardless of PrevMedium or CurrentMedium.
% if FirstReplace is false, LastMedia of every chamber is initialized to
% PrevMedium, which corresponds to the medium's index in the vector MediaInputs, NOT the value of the medium in MediaInputs.
PrevMedium = 1;

% Time base for feeding in hours
% Each time corresponds to one row in ExpMatrix
FeedTimeBases =    [2];
% Delays in the start of feeding in hours
FeedStartDelays =  [0];

% Time period for bottle switching, in hours
BottleTimeBase = 1000;
  
% Inputs where media bottles are connected.
% Each row is a diferent type of media.
% Each column corresponds to a different time.
% corresponds to MediaNames in generatemediaschedule
MediaInputs = [     
    3; % dmem
    4; % ligand
]; 
% Input purge time (minutes)
% To clear out tubes before first feed
InputPurgeTime = 0;

% Duration (sec) that reagent is flowed into chamber
ChFlowTime = 6; % seconds

% Input labels
InputLabels = {'','','dmem','ligand','','','','','','','','','','','',''};

% 3D Cell array with chambers to be fed at each time interval
% Each row is for a different feeding frequency, each column is for a
% different feeding volume (# of cycles).
% Third dimension is for the medium schedule, corresponding to the row of MediaSchedule
% Each element of the array is a vector of chamber numbers
load([exp_path ExpMatrixFile '.mat']);
szmatrix = size(ExpMatrix);
if (length(szmatrix) == 1)
    szmatrix = [szmatrix, 1, 1];
elseif (length(szmatrix) == 2)
    szmatrix = [szmatrix, 1];
end %if (length(szmatrix) == 1)

% Load media schedule matrix, schedule_time_base, schedule_length
% Each row in the schedule is a different medium schedule (condition),
% corresponding to the 3rd dimention of ExpMatrix
% Each column is a time point.
% The duration of a time point is equal to schedule_time_base
load([exp_path MediaScheduleFile '.mat']);
szschedule = size(MediaSchedule);
% infodlg('String', ['szschedule = ' num2str(szschedule)]);
NumConditions = min(szschedule(1), szmatrix(3));
if (szmatrix(3) ~= szschedule(1))
    str = ['Number of conditions in ExpMatrix does not match FeedSchedule' ...
        10 13 '#Conditions = ' num2str(szschedule(1)) ' from FeedSchedule' ...
        10 13 '#Conditions = ' num2str(szmatrix(3)) ' from ExpMatrix' ...
        10 13 10 13 'Choosing the smallest number!'];
    infodlg('Title', 'Warning', 'String', str);
end %if (szmatrix(3) ~= szschedule(1))

NumTimes = length(FeedTimeBases);
NumAmnts = length(FeedCycles);

% make matrix with same dimensions as expmatrix, where 1 indicates chambers
% exist at that point and 0 indicates no chambers for that point in the
% matrix. use this for determining if necessary to purge inputs/mixer, mux.
chamberexist = zeros(NumTimes, NumAmnts, NumConditions);
size(ExpMatrix)
size(chamberexist)
for tindex=1:NumTimes
    for aindex=1:NumAmnts
        for cindex=1:NumConditions
			if ~isempty(ExpMatrix{tindex, aindex, cindex})
                chamberexist(tindex, aindex, cindex)=1;
			end
        end
    end
end

% Open report file
log_file = [exp_path log_file];
if exist(log_file, 'file')
    write_header = 0;
else
    write_header = 1;
end %if exist(log_file, 'file')
logfid = fopen(log_file, 'at');
if write_header
    fprintf(logfid, '%% %s\n', datestr(clock));
    fprintf(logfid, '%% %s\n', log_file);
    fprintf(logfid, '%% %s\n', feed_file);
    sz = size(exp_description);
    for ii = 1:sz(1)
        fprintf(logfid, '%% %s\n', exp_description{ii});
    end %for ii
end %if write_header

% Write parameters to log file
fprintf(logfid, '%s\t----------------Script Started----------------------\n', datestr(clock));
fprintf(logfid, '\t\tSpeedFactor = %6.3f\n', SpeedFactor);
fprintf(logfid, '\t\tExpMatrixFile = %s.mat\n', ExpMatrixFile);
fprintf(logfid, '\t\tMediaScheduleFile = %s.mat\n', MediaScheduleFile);
fprintf(logfid, '\t\tFeedTimeBases = %6.3f\n', FeedTimeBases);
fprintf(logfid, '\t\tFeedStartDelays = %6.3f\n', FeedStartDelays);
fprintf(logfid, '\t\tFeedCycles = %s\n', num2str(FeedCycles));
fprintf(logfid, '\t\tFeedCycles0 = %s\n', num2str(FeedCycles0));
fprintf(logfid, '\t\tFeedPumpPeriods = %s\n', num2str(FeedPumpPeriods, '%6.3f '));
if AlternatePumpDir
    str = 'true';
else
    str = 'false';
end
fprintf(logfid, '\t\tAlternatePumpDir = %s\n', str);
if PurgeMux
    str = 'true';
else
    str = 'false';
end
fprintf(logfid, '\t\tPurgeMux = %s\n', str);
fprintf(logfid, '\t\tschedule_time_base = %6.2f\n', schedule_time_base);
fprintf(logfid, '\t\tschedule_length= %i\n', schedule_length);
fprintf(logfid, '\t\tMediaInputs = \n');
for ii = 1:length(MediaInputs(:, 1))
    fprintf(logfid, '\t\t\t%i --> %s\n', ii, num2str(MediaInputs(ii, :)));
end %for ii
fprintf(logfid, '\t\tBottleTimeBase = %6.3f\n', BottleTimeBase);
% fprintf(logfid, '\n');
fprintf(logfid, '\t\tChamber Matrix:\n');
fprintf(logfid, '\t\t\tPeriod (hrs)\t\tCycles\t\tSchedule\n');
for ii = 1:szmatrix(1)
    for jj = 1:szmatrix(2)
        for kk = 1:szmatrix(3)
            fprintf(logfid, '\t\t\t%4.2f\t\t%i\t\t%i', FeedTimeBases(ii), FeedCycles(jj), kk);
            fprintf(logfid, '\t%s\n', num2str(ExpMatrix{ii, jj, kk}));
        end %for kk
    end %for jj
end %for ii
str = '';
nl = length(InputLabels);
for ii = 1:nl
    if ii == nl
        ss = '';
    else
        ss = ', ';
    end %if ii == nl
    str = [str InputLabels{ii} ss];
    eval(['set(CCChs.edit' num2str(ii + 43) ', ''String'', ''' InputLabels{ii} ''');']);
end %for ii
fprintf(logfid, '\t\tMediaLabels = %s\n', str);

fprintf(logfid, '\tRun Info:\n');
sz = size(run_info);
for ii = 1:sz(1)
    fprintf(logfid, '\t%s\n', run_info{ii});
end %for ii
fprintf(logfid, '\t\t----------------------------\n');

% Load last feeding times from previous run of the script
data_file = [exp_path 'ScriptData_valvesOnly.mat'];
if exist(data_file, 'file')
    load(data_file, 'Tlastfeed', 'feed_cnt', 'Tlastbottle', 'bottle_idx', 'Tschedulestart', 'schedule_idx', 'first_feed', 'LastMedia', 'Tlastco2', 'InputsPurged', 'PumpForward', 'PreviousMedium');
else
    % Vector with last feeding times.
    % Each element is for a different feeding period
    Tlastfeed = now*ones(1, NumTimes) - FeedTimeBases/24 + FeedStartDelays/24;
    % Feed cycle counter
    feed_cnt = 1;
    InputsPurged = 0;

    % Time of last bottle switch
    Tlastbottle = now + min(FeedStartDelays)/24;
    % Index of bottles being used (indexes the column of MediaInputs)
    bottle_idx = 1;
    % Time of last media schedule switch
    Tschedulestart = now + min(FeedStartDelays)/24;
    % Time of last CO2 and temp. readings
    Tlastco2 = now - 2;
    % Index of time in the schedule (indexes column of FeedSchedule)
    schedule_idx = 1;
    % Flag to alternate pumping direction
    PumpForward = ones(96, 1);
    PreviousMedium=999; %added when the one at ~385 was commented out
    % Storage of media used for last feed
    % Initialized to medium specified by 1st entry in the media schedule
    LastMedia = zeros(96, 1);
    if PrevMediaSchedule
        for ii = 1:NumTimes
            for jj = 1:NumAmnts
                for kk = 1:NumConditions
                    for ch = ExpMatrix{ii, jj, kk}
                        LastMedia(ch) = fix(MediaSchedule{kk}(1));
                        if LastMedia(ch) > 100
                            LastMedia(ch) = LastMedia(ch) - 100;
                        end %if LastMedia(ch) > 100
                    end %for ch
                end %for kk
            end %for jj
        end %for ii
    else
        if ~FirstReplace
            LastMedia = ones(96, 1)*PrevMedium;
        end % if FirstReplace
    end % if PrevMediaSchedule
    
    % Indicator of whether this is the first feed ever
    first_feed = 1;
end %if exist(data_file, 'file')
fprintf(logfid, '\t\tTlastfeed = \n');
nt = length(Tlastfeed);
for ii = 1:nt
    fprintf(logfid, '\t\t\t%s\n', datestr(Tlastfeed(ii)));
end %for ii

fprintf(logfid, '\t\tTschedulestart = %s, schedule_idx = %i\n', datestr(Tschedulestart), schedule_idx);
fprintf(logfid, '\t\tLastMedia = %s\n', num2str(LastMedia', '%i '));
fprintf(logfid, '\t\tPumpDirection = %s\n', num2str(PumpForward', '%i '));
fprintf(logfid, '\t\tTlastbottle = %s, bottle_idx = %i\n', datestr(Tlastbottle), bottle_idx);
fprintf(logfid, '\t\t----------------------------\n');

try
    % Main cycle
    %------------
    while ~CCCdata.Scr.Stop && ~exist([exp_path 'StopScript'], 'file')
        
        % Switch bottles if necessary
        dt_bottle = (now - Tlastbottle)*24*SpeedFactor;
        if dt_bottle >= BottleTimeBase
            Tlastbottle = now;
            szinputs = size(MediaInputs);
            li = szinputs(2);
            old_idx = bottle_idx;
            if bottle_idx < li
                bottle_idx = bottle_idx + 1;
            else
                bottle_idx = 1;
            end
            %fprintf(logfid, '%s\tSwitched media bottles from inputs %s to inputs %s, dt_bottle = %6.2f\n', datestr(clock), num2str(MediaInputs(:, old_idx)', '%i '), num2str(MediaInputs(:, bottle_idx)', '%i '), dt_bottle);
            save(data_file, 'Tlastfeed', 'feed_cnt','Tlastbottle', 'bottle_idx', 'Tschedulestart', 'schedule_idx', 'first_feed', 'LastMedia', 'InputsPurged', 'PumpForward', 'PreviousMedium');
       end

        % Advance schedule index if necessary
        old_schedule_idx = schedule_idx;
        dt_schedule = (now - Tschedulestart)*24*SpeedFactor;
        schedule_idx = 1 + fix(floor(dt_schedule/schedule_time_base));
        if (schedule_idx > schedule_length)
            schedule_idx = schedule_length;
        end
        if schedule_idx ~= old_schedule_idx
                fprintf(logfid, '%s\tNew schedule index: schedule_idx = %i, dt_schedule = %6.2f\n', datestr(clock), schedule_idx, dt_schedule);
        end

        % Purge input tubes really well if this is the first feed
        % to eliminate any water and/or air in them
        if ~InputsPurged && (InputPurgeTime > 0)
            InputsPurged = 1;
            clear('PurgeInputsExec');
            PurgeInputsExec;
            save(data_file, 'Tlastfeed', 'feed_cnt','Tlastbottle', 'bottle_idx', 'Tschedulestart', 'schedule_idx', 'first_feed', 'LastMedia', 'InputsPurged', 'PumpForward', 'PreviousMedium');
		end
		
        % Feed chambers
        for time_idx = 1:NumTimes
            % Compute elapsed times since last feeding rounds, in hours
            dt_feed = (now - Tlastfeed)*24*SpeedFactor;
            scr_check4stop;
            if dt_feed(time_idx) >= FeedTimeBases(time_idx)
                Tlastfeed(time_idx) = now;
                fprintf(logfid, '%s\tStart feeding chambers with period %6.2f hrs, dt_feed = %6.2f\n', datestr(clock), FeedTimeBases(time_idx), dt_feed(time_idx));
                open_valve('Waste1Up');
                open_valve('Waste1Dn');
                for condition_idx = 1:NumConditions
                    scr_check4stop;
                    CurrentMedium = MediaSchedule{condition_idx}(schedule_idx);
                    % Check if a # of cycles was specified in the
                    % medium schedule (as 3 digits to the right of
                    % the decimal point)
                    CycleSpec = int16(1000*(CurrentMedium - fix(CurrentMedium)));
                    CurrentMedium = fix(CurrentMedium);
                    % Check if there's a replace flag in the medium
                    % schedule
                    if CurrentMedium >= 100
                        CurrentMedium = CurrentMedium - 100;
                        ReplaceFlag = 1;
                    else
                        ReplaceFlag = 0;
                    end %if CurrentMedium
                    if CurrentMedium ~= 0 && sum(chamberexist(time_idx, :, condition_idx))>0 % if not nonfeeding and if expmatrix isn't empty along all renewal rates at this frequency and mediaschdule
                        CurrentMediaInput = MediaInputs(CurrentMedium, bottle_idx);
                        % Purge the inputs if medium changed from previous
                        % condition in the loops
                        if CurrentMedium ~= PreviousMedium
                            close_inputs;
                            fprintf(logfid, '%s\t\tPurging inputs/mixer, mux, cell input. CurrentMedium = %i, condition_idx = %i.\n', datestr(clock), CurrentMedium, condition_idx);
                            open_valve('InPurge');
                            open_valve('MixerOut');
                            % Purge inputs
                            open_valve('Flush');
                            scr_message('Purging inputs for 10sec');
                            pause(10); %was 15
                            close_valve('Flush');
                            % Purge cell input line
                            open_pump;
                            open_valve('CellsIn');
                            scr_message('Purging Cell input line for 2sec');
                            pause(2);
                            close_valve('CellsIn');
                            pause(1);
                            % Purge MUX
                            set_mux(97, 1, 1, 1, 0, 0);
                            scr_message('Purging MUX for 15sec')
                            pause(15); %was 20
                            set_mux(99, 1, 1, 0, 1, 1);
                            close_pump;
                            close_valve('MixerOut');
                            close_valve('InPurge');
                            pause(1);
                        end %if CurrentMedium
                        flush_mixer(CurrentMediaInput, 5);
                        open_inputs(CurrentMediaInput);
                        pause(1);
                        for amnt_idx = 1:NumAmnts
                            scr_check4stop;
                            if first_feed
                                ncycles = FeedCycles0(amnt_idx);
                            else
                                ncycles = FeedCycles(amnt_idx);
                            end %if first_feed
                            
                            
                            if CycleSpec > 0
                                PmpCycles = CycleSpec;
                            else
                                PmpCycles = ncycles;
                            end %if CycleSpec > 0
                            chambers = ExpMatrix{time_idx, amnt_idx, condition_idx};
                            if ~isempty(chambers)
                                NumChambers = length(chambers);
                                chcnt = 1;
                                for chamber = chambers
                                    scr_check4stop;
                                    
                                    %%%% RK 24.11.2013
                                    if PmpCycles < 0
                                        PumpForward(chamber) = 0;
%                                         PmpCycles = abs(PmpCycles);
%                                         %moved to pump function call to
%                                         prevent problem with 2nd chamber
                                    end
                                    
                                    % Check if medium changed from previous feeding
                                    % round, for this chamber
                                    ReplaceMedium = 0;
                                    if (CurrentMedium ~= LastMedia(chamber))
                                        DifferentMediumFlag = 1;
                                    else
                                        DifferentMediumFlag = 0;
                                    end
                                    if (PmpCycles == 999) || xor(ReplaceFlag, DifferentMediumFlag)
                                        ReplaceMedium = 1;
                                    end %if (ncycles ...
                                    close_pump;
                                    set_mux(chamber, 1, 1, 1, 0, 0);
                                    % Flush chamber pair
                                    if rem(chcnt, 2)
                                        open_valve('MixerOut');
                                           if ReplaceMedium
                                               scr_message('Charging flush line for 10sec');
                                               open_valve('Flush');
                                               pause(10); %was 16
                                               close_valve('Flush');
                                           end
                                        open_pump;
                                        if CurrentMedium ~= PreviousMedium
                                            flsht = 5; %was 15
                                        else
                                            flsht = 2; %was 5
                                        end
                                        if ~PumpForward(chamber)
                                            flsht = flsht + 10;
                                        end
                                        scr_wait(0, flsht, 'Flushing chamber pair');
                                        close_pump;
                                    end %if rem(chcnt, 2)
                                    open_valve('ChIn');
                                    open_valve('ChOut');
                                    set_mux(chamber, 1, 1, 0, 0, 0);
                                    % Replace medium in chamber or pump a new dose
                                    if ReplaceMedium
                                        fprintf(logfid, '%s\t\tReplacing medium in chamber %i: media input = %i (%s)\n', datestr(clock), chamber, CurrentMediaInput, InputLabels{CurrentMediaInput});
                                        scr_message(['Replacing medium in ch ' num2str(chamber)]);
                                        close_valve('Flush');
                                        pause(0.2);
                                        open_valve('MixerOut');
                                        open_pump;
                                        pause(ChFlowTime); % was 6
                                        close_pump;
                                        close_valve('Flush');
                                    else
                                        if PumpForward(chamber)
                                            str = 'forward';
                                            close_valve('Flush');
                                            open_valve('MixerOut');
                                            PmpC = PmpCycles;
                                        else
                                            str = 'backward';
                                            close_valve('MixerOut');
                                            open_valve('Flush');
                                            PmpC = fix(1.6*PmpCycles);
                                        end
                                        pump_direction(str);
                                        fprintf(logfid, '%s\t\tFeeding chamber %i: media input = %i (%s), cycles = %i, pmp period = %6.3fsec, direction = %s\n', datestr(clock), chamber, CurrentMediaInput, InputLabels{CurrentMediaInput}, PmpCycles, FeedPumpPeriods(amnt_idx), str);
                                        scr_message(['Feeding chamber ' num2str(chamber) ' with ' num2str(ncycles) ' cycles from input ' num2str(CurrentMediaInput) ' (period = ' num2str(FeedTimeBases(time_idx)) 'hrs)']);
                                        pause(1);
                                        pump(PmpC, FeedPumpPeriods(amnt_idx), 1);
                                        if ~PumpForward(chamber)
                                            close_valve('Flush');
                                        end
                                        if AlternatePumpDir
                                            PumpForward(chamber) = ~PumpForward(chamber);
                                        end
                                    end %if ReplaceMedium
                                    close_valve('ChIn');
                                    close_valve('ChOut');
                                    pause(0.5);

                                    chcnt = chcnt + 1;
                                    LastMedia(chamber) = CurrentMedium;
                                    scr_check4stop;
                                end %for chamber
                                close_valve('ChIn');
                                close_valve('ChOut');
                                close_valve('Flush');
                                close_valve('MixerOut');
                                close_pump;
                            end %if ~isempty(chambers)
                        end %for amt_idx
                        PreviousMedium = CurrentMedium;
                    end %if CurrentMedium~=0 ...
                end %for condition_idx
                close_inputs;
                first_feed = 0;
                % Save last feeding times in case script is stopped
                save(data_file, 'Tlastfeed', 'feed_cnt','Tlastbottle', 'bottle_idx', 'Tschedulestart', 'schedule_idx', 'first_feed', 'LastMedia', 'InputsPurged', 'PumpForward', 'PreviousMedium');
                fprintf(logfid, '%s\tEnded feeding chambers with period %6.2f hrs\n', datestr(clock), FeedTimeBases(time_idx));
               
                % Reset time_idx to never skip the most frequent feedings
                time_idx = 1;
                feed_cnt = feed_cnt + 1;
                if PurgeMux
                    fprintf(logfid, '%s\tPurging MUX\n', datestr(clock));
                    % Leave all channels filled with purging medium
                    purge_inputs(10); %was 15
                    pause(1);
                    purge2muxflush(97, 20); %was 30
                    PreviousMedium = 999;
                end %if PurgeMux
            end %if dt_feed
        end %for time_idx
        set_mux(99, 1, 1, 1, 1, 1);

        % Flush log file by closing it
        fclose(logfid);
        % Re-open log file
        logfid = fopen(log_file, 'at');

        % Wait for 10sec
        dt_feed_hrs = fix(dt_feed);
        dt_feed_min = (dt_feed - dt_feed_hrs)*60;
        dt_feed_str = '';
        for ii = 1:NumTimes
            dt_feed_str = [dt_feed_str num2str(dt_feed_hrs(ii), '%2.0fh ') ...
                num2str(dt_feed_min(ii), '%2.0fm') ' - '];
        end
        
        dt_schedule_hrs = fix(dt_schedule);
        dt_schedule_min = (dt_schedule - dt_schedule_hrs)*60;
        dt_schedule_str = [num2str(dt_schedule_hrs, '%2.0fh') num2str(dt_schedule_min, '%2.0fm')];
        scr_wait(0, 10, ['Ended one cycle.' 10 13 ...
            'dt_feed=' dt_feed_str ', schedule_idx=' num2str(schedule_idx) ...
            ', dt_schedule=' dt_schedule_str]);
        save(data_file, 'Tlastfeed', 'feed_cnt', 'Tlastbottle', 'bottle_idx', 'Tschedulestart', 'schedule_idx', 'first_feed', 'LastMedia', 'Tlastco2', 'InputsPurged', 'PumpForward', 'PreviousMedium');

%         % Re-open log file
%         logfid = fopen(log_file, 'at');
        scr_check4stop;
    end %while
catch
    str = lasterr;
    fprintf(logfid, '%s\tERROR!!!!  %s\n', datestr(clock), str);
    infodlg('Title', 'Feed Chambers', 'String', str);
end

% Close all valves for safety
set_mux(99, 1, 1, 1, 1, 1);
close_valve('ChIn');
close_valve('ChOut');
close_valve('Flush');
close_valve('MixerOut');
close_pump;
close_inputs;

if CCCdata.Scr.Stop
    fprintf(logfid, '%s\tScript stopped!!!!\n', datestr(clock));
elseif exist([exp_path 'StopScript'], 'file')
    fprintf(logfid, '%s\tScript stopped through StopScript file!!!!\n', datestr(clock));
else
    fprintf(logfid, '%s\tScript ended!!!!\n', datestr(clock));
end

fclose(logfid);

