%%% 
% This script perform sequential chamber operations, to i.e. prepare chip 
% with pluronic and fibronectin treatment 
%%%

% Program below will perform pluronic and fibronectin treatment
% step #		1			2			3		4			5			
InputName =     {'PBS',    'PBS',		'Pluronic', 'pbsFlAC',	'PBS',  'Fibronectin',  'pbsFlAC',	'PBS',  'dmemFlAC',	'dmem'};
InputNumber =   {0,         0,    	   	1,          0,    	   	0,      2,             0,    	   	0,      3,    	   	3};      
toChambers =    {1,         1,    	  	0,          0,    	  	0,      0,              0,    	  	0,      0,    	   	1};         
toCellsIn =     {1,         1,    	  	1,          1,    	  	0,      0,              1,    	  	0,      1,    	   	1};         
startFlTime =   {1,         0,    	   	1,         1,          1,      0.5,            1,          .1,      1,    	   	1}; % min 
incubTime =     {0,         0,    	  	60,         0,    	  	0,      0,             0,    	  	480,    0,    	   	1}; % min   
flTime =        {5*60,      0,   		4,          300,   	 	4,      3,              300,   	 	8,     300,   	  	10}; % sec  
chTime =        {0,         10*60,    	0,          0,    	  	0,      0,              0,    	  	0,      0,    	   	12}; % sec  
cinTime =       {10,       	0,    	 	30,         30,    	 	30,     0,              20,    	 	10,      10,    	  	10}; % sec  
chambers =      {97,      	97, 		1:96,       97, 		1:96,   1:96,           97, 		1:96,   97, 	    1:96};       % 97 = all chambers at once
repeats =       {1,         1,    	  	1,          1,    	  	2,      1,              1,    	  	1,      1,    	   	1};         
wastesClosed =  {1,         1,    	  	0,          0,    	  	0,      0,              0,    	  	0,      0,    	   	0};         
vals = [5 6];  % vals = [3:5] if chip is already debubbled with PBS


for i = vals
    scr_check4stop;
    
    close_valve('ChIn');
    close_valve('ChOut');
    set_mux(99, 1, 1, 1, 0, 1);

    close_inputs;
    close_pump;
    if (InputNumber{i}) == 100
        open_valve('CellsIn')
        open_pump;

    else
        open_inputs(InputNumber{i}) % PBS 
        open_valve('MixerOut');
    end
    open_valve('Flush');
    scr_wait(startFlTime{i}, 0, ['Flush ' InputName{i}]);
    
    close_valve('Flush')
    open_valve('Waste1Dn')
    open_valve('Waste1Up')
    
    % wash CellsIn
    if toCellsIn{i} 
        set_mux(97, 1, 1, 1, 1, 1);
        open_pump;
        open_valve('CellsIn')
        scr_wait(0,cinTime{i}, [InputName{i} ' to CellsIn']);
        close_valve('CellsIn');
    end

    for k = 1:repeats{i}
        scr_check4stop;          
        
        if wastesClosed{i}
            close_valve('Waste1Up');
            close_valve('Waste2Up');
            close_valve('Waste1Dn');
            close_valve('Waste2Dn');
        else
            open_valve('Waste1Up');
            open_valve('Waste1Dn');
        end
            
        for c = chambers{i}
            scr_check4stop;
            set_mux(c, 1, 1, 1, 0, 1);
            open_pump;
            scr_wait(0, flTime{i}, [InputName{i} ' flush']);

            if toChambers{i}
                set_mux(c, 1, 1, 0, 0, 0);
                scr_wait(0, 2, ['Pause for chamber valve']);
                open_valve('ChIn');
                open_valve('ChOut');
                scr_wait(0, chTime{i}, [InputName{i} ' chamber']);
                close_valve('ChIn');
                close_valve('ChOut');
                scr_wait(0, 1, ['Pause for chamber valve']);
                set_mux(99, 1, 1, 0, 0, 0);
            end

        end
        set_mux(99, 1, 1, 1, 0, 1);
        close_pump;
        scr_wait(incubTime{i}, 0, ['Incubating ' InputName{i}]);
    end

end


