%%% GenerateExpMatrix
% Generate a random or sequential assignment of experimental conditions to chamber pairs
% R. Gmez-Sjberg
%%%

% Log file name
log_file = 'ExpMatrix-log';
% Matrix file name
MatrixFile = 'ExpMatrix';

% Name of variable to save matrix to
var_name = 'ExpMatrix';

sz = [1 2 2]; % frequencies x renewal rates x media schedules
% Number of chamber *PAIRS* to assign to each condition
% Leave empty to have equal # of pairs for each condition
% numchs = [];
numchs = zeros(sz);

% deliver dmem to 2 chamber pairs (ch #1-4) every 2hrs with 30% replacement
% deliver ligand to 3 3 chamber pairs (ch #5-10), every 2hrs with 100%
% replacement: 
numchs(1, 1, 1) = 2;
numchs(1, 2, 2) = 3;

% Names of the conditions
cond_names = {'Frequency', 'Renewal', 'Schedule'};
% Labels for each column of the matrix
labels_x = {'2hr'};        
labels_y = {'30%','100%'};
labels_z = {'dmem','ligand'};

% Assign chambers randomly or manually?
randomize = false;

% Manually assigned chambers
matrix = cell(sz);

% Chamber pairs to assign
% A chamber pair is identified by the number of the 1st chamber in the pair
%     Sch1 = [ 1:6, 9:40, 43:50, 53:72, 75:96];
%     Sch2 = [ 7, 8, 41, 42, 51, 52, 73, 74];
% Sch3 = [ 7,  8, 11, 12, 19, 20, 41, 42, 51, 52, 59, 60, 69, 70, 71, 72, 79, 80, 81, 82, 85, 86, 95, 96];
% Sch4 = [15, 16, 27, 28, 31, 32, 37, 38, 45, 46, 49, 50, 55, 56, 61, 62, 77, 78, 83, 84, 87, 88, 93, 94];
% idxSch1 = find(rem(Sch1, 2));
% idxSch2 = find(rem(Sch2, 2));
% idxSch3 = find(rem(Sch3, 2));
% idxSch4 = find(rem(Sch4, 2));
% chpairs = {sort(Sch1(idxSch1)); sort(Sch2(idxSch2)); sort(Sch3(idxSch3)); sort(Sch4(idxSch4))};

chpairs = {[1 3]; [5 7 9]};

disp(' ');
disp(' ');
disp(['Generating experimental matrix: ' log_file]);
% Create a report file
log_file = [CCCdata.DataPath log_file '.txt'];
logfid = fopen(log_file, 'wt');
fprintf(logfid, '%% %s\n', datestr(clock));
fprintf(logfid, '%% %s\n', log_file);
if randomize
    str = 'Random';
else
    str = 'Manual';
end
fprintf(logfid, '%% %s assignment of chamber pairs to each experimental condition\n', str);
fprintf(logfid, '%% Experimental matrix: %i x %i x %i\n', sz(1), sz(2), sz(3));
fprintf(logfid, '%% Experimental matrix file name: %s.mat\n', MatrixFile);
fprintf(logfid, '%% %s\t%s\t%s\n', cond_names{1}, cond_names{2}, cond_names{3});
    
% Find max length of labels
maxl_x = 0;
for ii = 1:sz(1)
    ll = length(labels_x{ii});
    if ll > maxl_x
        maxl_x = ll;
    end
end
maxl_y = 0;
for ii = 1:sz(2)
    ll = length(labels_y{ii});
    if ll > maxl_y
        maxl_y = ll;
    end
end
maxl_z = 0;
for ii = 1:sz(3)
    ll = length(labels_z{ii});
    if ll > maxl_z
        maxl_z = ll;
    end
end
spc_vec = '                                                          ';

try
    % Draw chamber pairs at random, and assign them to each condition
    if isempty(numchs)
        % Number of chamber *pairs* in each condition
        xx = length(chpairs);
        nch = 0;
        for ii = 1:xx
            nch = nch + length(chpairs{ii});
        end
        npairs0 = floor(nch / prod(sz));
    end
    for ii = 1:sz(1)
        for kk = 1:sz(3)
            for jj = 1:sz(2)
                % Use mm to link the ROWS in chpairs to a particular
                % dimension in the matrix (mm=ii, or mm=kk, or mm=jj)
                mm = kk; nn = 1;  
                if ~isempty(numchs)
                    npairs = numchs(ii, jj, kk) - length(matrix{ii, jj, kk})/2;
                else
                    npairs = npairs0 - length(matrix{ii, jj, kk})/2;
                end
                if npairs > 0
                    for ll = 1:npairs
                        lp = length(chpairs{mm, nn});
                        if lp == 0
                            error('Not enough chambers to assign to all conditions');
                        end
                        % Select a pair of chambers
                        if randomize
                            idx = fix(lp*rand + 1);
                        else
                            idx = 1;
                        end
                        % If random, flip a coin to select order of chambers in the pair
                        if randomize && (rand > 0.5)
                            matrix{ii, jj, kk} = [matrix{ii, jj, kk}, chpairs{mm, nn}(idx) + 1, chpairs{mm, nn}(idx)];
                        else
                            matrix{ii, jj, kk} = [matrix{ii, jj, kk}, chpairs{mm, nn}(idx), (chpairs{mm, nn}(idx) + 1)];
                        end %if rand
                        % Eliminate the chosen pair from the list
                        chpairs{mm, nn} = chpairs{mm, nn}([1:(idx-1), (idx+1):end]);
                    end %for ll
%                 else
%                     matrix{ii, jj, kk} = [];
                end %if npairs
                if ~isempty(matrix{ii, jj, kk})
                    % Pad the label strings with spaces at the end so they
                    % are all the same length
                    str_x = labels_x{ii};
                    if length(str_x) < maxl_x
                        str_x = [str_x spc_vec(1:(maxl_x - length(str_x)))];
                    end
                    str_y = labels_y{jj};
                    if length(str_y) < maxl_y
                        str_y = [str_y spc_vec(1:(maxl_y - length(str_y)))];
                    end
                    str_z = labels_z{kk};
                    if length(str_z) < maxl_z
                        str_z = [str_z spc_vec(1:(maxl_z - length(str_z)))];
                    end
                    fprintf(logfid, '%s\t%s\t%s', str_x, str_y, str_z);
                    % Write condition to the log file
                    fprintf(logfid, '\t%s\n', num2str(matrix{ii, jj, kk}, '\t%i'));
                end %if ~isempty...                
            end %for jj
        end %for kk
    end %for ii
    % Save experimental matrix
    eval([var_name '= matrix;']);
    save([CCCdata.DataPath MatrixFile '.mat'], var_name);

    figure(10);
    clf;
    cmap = jet(sz(3));
    symb = {'o', '^', '+', 'v', 's', '*', '>', 'd', '<'};
    % Write table again, but listing individual chambers
    fprintf(logfid, '\n\n\n%Frequency\tRenewal\tSchedule\tCh#\n');
    chambers = [];
    for ii = 1:sz(1)
        for jj = 1:sz(2)
            for kk = 1:sz(3)
                nch = length(matrix{ii, jj, kk});
                for ll = 1:nch
                    ch = matrix{ii, jj, kk}(ll);
                    fprintf(logfid, '%s\t%s\t%s\t%s\n', labels_x{ii}, labels_y{jj}, labels_z{kk}, num2str(ch, '%i'));
                    if ch < 49
                        yy = 1;
                        xx = 2*(49 - ch);
                    else
                        yy = 10;
                        xx = 2*(ch - 48);
                    end
                    text(xx, yy+0.4, num2str(ch), 'FontSize', 6);
                    hp = plot(xx, yy, symb{jj});
                    hold on;
                    set(hp, 'MarkerEdgeColor', cmap(kk, :));
                    set(hp, 'DisplayName', [labels_x{ii} ', ' labels_y{jj} ', ' labels_z{kk}]);
                    chambers = [chambers; ch];
                end %for ll
            end %for kk
        end %for jj
    end %for ii
    axis([0 100 0 12]);
    chambers = sort(chambers);
    if length(chambers) ~= 96
%         infodlg('Title', 'Exprimental Matrix', 'String', 'Missing chambers!!!');
    elseif sum(chambers - [1:96]') ~= 0
        infodlg('Title', 'Exprimental Matrix', 'String', 'Duplicated chambers!!!');
    end

catch
    str = ['Error!' 10 13 lasterr]; % 10 13 'idx = ' num2str(idx) '  ii = ' num2str(ii) ' jj = ' num2str(jj) '  kk= ' num2str(kk) '  ll = ' num2str(ll) 10 13 '  npairs= ' num2str(npairs) ' lp= ' num2str(lp)];
    infodlg('Title', 'Chamber Assignment', 'String', str);
end %try

disp('Closing log file');
fclose(logfid);
