%%% LoadSequentialTreatmentFromCSV
% Load sequential treatment from csv
% Philip Dettinger
% Adapted from R. Kelloggs SequentialTreatments.m
%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% LOAD CSV WITH RELEVANT STEPS
% Get CSV
[filename, pathname] = uigetfile('.csv')

% Import and Split
data1 = importdata(fullfile(pathname,filename),'\t');
expression = ';';
splitStr = regexp(data1,expression,'split');
StepNum = length(splitStr{1,1}) - 1;
Steps = 2:StepNum;


% Interpret
InputName = splitStr{1,1}(1,Steps);  
InputNumber =   splitStr{2,1}(1,Steps);
toChambers =    splitStr{3,1}(1,Steps);
toCellsIn =    splitStr{4,1}(1,Steps);
startFlTime =   splitStr{5,1}(1,Steps);
incubTime =     splitStr{6,1}(1,Steps);
flTime =        splitStr{7,1}(1,Steps);
chTime =       splitStr{8,1}(1,Steps);
cinTime =      splitStr{9,1}(1,Steps);
chambers =      splitStr{10,1}(1,Steps);
repeats =       splitStr{11,1}(1,Steps);
wastesClosed =  splitStr{12,1}(1,Steps);
vals = [1:StepNum - 2];

for i = vals
    scr_check4stop;
    
    close_valve('ChIn');
    close_valve('ChOut');
    set_mux(99, 1, 1, 1, 0, 1);

    close_inputs;
    close_pump;
    if (InputNumber{i}) == 100
        open_valve('CellsIn')
        open_pump;

    else
        open_inputs(InputNumber{i}) % PBS 
        open_valve('MixerOut');
    end
    open_valve('Flush');
    scr_wait(startFlTime{i}, 0, ['Flush ' InputName{i}]);
    
    close_valve('Flush')
    open_valve('Waste1Dn')
    open_valve('Waste1Up')
    
    % wash CellsIn
    if toCellsIn{i} 
        set_mux(97, 1, 1, 1, 1, 1);
        open_pump;
        open_valve('CellsIn')
        scr_wait(0,cinTime{i}, [InputName{i} ' to CellsIn']);
        close_valve('CellsIn');
    end

    for k = 1:repeats{i}
        scr_check4stop;          
        
        if wastesClosed{i}
            close_valve('Waste1Up');
            close_valve('Waste2Up');
            close_valve('Waste1Dn');
            close_valve('Waste2Dn');
        else
            open_valve('Waste1Up');
            open_valve('Waste1Dn');
        end
            
        for c = chambers{i}
            scr_check4stop;
            set_mux(c, 1, 1, 1, 0, 1);
            open_pump;
            scr_wait(0, flTime{i}, [InputName{i} ' flush']);

            if toChambers{i}
                set_mux(c, 1, 1, 0, 0, 0);
                scr_wait(0, 2, ['Pause for chamber valve']);
                open_valve('ChIn');
                open_valve('ChOut');
                scr_wait(0, chTime{i}, [InputName{i} ' chamber']);
                close_valve('ChIn');
                close_valve('ChOut');
                scr_wait(0, 1, ['Pause for chamber valve']);
                set_mux(99, 1, 1, 0, 0, 0);
            end

        end
        set_mux(99, 1, 1, 1, 0, 1);
        close_pump;
        scr_wait(incubTime{i}, 0, ['Incubating ' InputName{i}]);
    end

end


















%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  HERE BE MONSTERS

% data = csvread(fullfile(pathname,filename))
% type(fullfile(pathname,filename))
% 
% 
% FID = fopen(fullfile(pathname,filename))
% data2 = textscan(FID,Frmt)
% ST = fclose(FID)
% 
% semicolons = ';';
% 
% firstsplit = regexp(data1,semicolons)
% 
% Frmt = '%s'
% for i = 1:length(firstsplit{1,1}) - 1
%     Frmt = horzcat(Frmt,' %s')
% end

